# File: app/__init__.py

import os
import json
import importlib
import time
import logging
import re
from datetime import datetime

from dotenv import load_dotenv, find_dotenv
from flask import Flask, request
from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate
from flask_login import LoginManager
from flask_wtf.csrf import CSRFProtect
from alembic.config import Config as AlembicConfig
from alembic import command

db            = SQLAlchemy()
migrate       = Migrate()
login_manager = LoginManager()
csrf          = CSRFProtect()


def run_autogen_and_upgrade(app, migrations_dir):
    ini_path = os.path.join(migrations_dir, 'alembic.ini')
    cfg = AlembicConfig(ini_path)
    cfg.set_main_option('script_location', migrations_dir)
    cfg.set_main_option('sqlalchemy.url', app.config['SQLALCHEMY_DATABASE_URI'])

    versions_dir = os.path.join(migrations_dir, 'versions')
    before = set(os.listdir(versions_dir)) if os.path.isdir(versions_dir) else set()

    # generate an autogen revision
    try:
        command.revision(cfg, message='autogen', autogenerate=True)
        app.logger.info("🛠️  Alembic autogen revision created.")
    except Exception as e:
        app.logger.info(f"🛠️  No schema changes detected: {e}")

    # remove any empty migrations
    after = set(os.listdir(versions_dir)) if os.path.isdir(versions_dir) else set()
    for fn in after - before:
        path = os.path.join(versions_dir, fn)
        text = open(path, 'r').read()
        if not re.search(r'op\.(?:add_column|create_table|alter_column|drop_column)', text):
            os.remove(path)
            app.logger.info(f"🛠️  Deleted empty migration {fn}")
        else:
            app.logger.info(f"🛠️  Keeping migration {fn}")

    # apply remaining migrations
    try:
        command.upgrade(cfg, 'head')
        app.logger.info("🛠️  Alembic upgrade to head succeeded.")
    except Exception as e:
        app.logger.error(f"🛠️  Alembic upgrade failed: {e}")
        raise


def create_app():
    if find_dotenv():
        load_dotenv(find_dotenv(), override=True)

    project_root   = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    migrations_dir = os.path.join(project_root, 'migrations')
    static_folder  = os.path.join(project_root, 'static')

    app = Flask(__name__, static_folder=static_folder, static_url_path='/static')

    handler = logging.StreamHandler()
    handler.setLevel(logging.INFO)
    app.logger.setLevel(logging.INFO)
    app.logger.addHandler(handler)
    app.config.from_object('app.config.Config')

    csrf.init_app(app)
    db.init_app(app)
    migrate.init_app(app, db, directory=migrations_dir)
    login_manager.init_app(app)
    login_manager.login_view = 'auth.login'

    with app.app_context():
        app.logger.info("🛠️  Running Alembic autogen & upgrade…")
        run_autogen_and_upgrade(app, migrations_dir)

    from .errors import bp as errors_bp  # noqa: E402
    app.register_blueprint(errors_bp)
    from .routes import init_app as register_core_routes  # noqa: E402
    register_core_routes(app)
    app.logger.info("✔️ Registered core routes")

    plugins_dir = os.path.join(project_root, 'plugins')
    for name in sorted(os.listdir(plugins_dir)):
        manifest = os.path.join(plugins_dir, name, 'plugin.json')
        if not os.path.isfile(manifest):
            continue

        errors = []
        meta = json.load(open(manifest))

        for model_path in meta.get('models', []):
            try:
                importlib.import_module(model_path)
            except Exception as e:
                errors.append(f"model '{model_path}': {e}")
        for sp in meta.get('subplugins', []):
            for model_path in sp.get('models', []):
                try:
                    importlib.import_module(model_path)
                except Exception as e:
                    errors.append(f"subplugin model '{model_path}': {e}")

        ul = meta.get('user_loader')
        if ul:
            try:
                fn = getattr(importlib.import_module(ul['module']), ul['callable'])
                fn(login_manager)
            except Exception:
                pass

        rt = meta.get('routes')
        if rt:
            try:
                bp = getattr(importlib.import_module(rt['module']), rt['blueprint'])
                app.register_blueprint(bp, url_prefix=rt.get('url_prefix'), strict_slashes=False)
            except Exception as e:
                errors.append(f"routes '{rt['module']}': {e}")
        for sp in meta.get('subplugins', []):
            rt = sp.get('routes')
            if rt:
                try:
                    bp = getattr(importlib.import_module(rt['module']), rt['blueprint'])
                    app.register_blueprint(bp, url_prefix=rt.get('url_prefix'), strict_slashes=False)
                except Exception as e:
                    errors.append(f"subplugin routes '{rt['module']}': {e}")

        cli = meta.get('cli')
        if cli:
            try:
                fn = getattr(importlib.import_module(cli['module']), cli['callable'])
                app.cli.add_command(fn)
            except Exception as e:
                errors.append(f"cli '{cli['module']}': {e}")

        for tg in meta.get('template_globals', []):
            try:
                mod_name, fn_name = tg['callable'].rsplit('.', 1)
                fn = getattr(importlib.import_module(mod_name), fn_name)
                app.jinja_env.globals[tg['name']] = fn
            except Exception as e:
                errors.append(f"template_global '{tg}': {e}")

        if errors:
            app.logger.error(f"Plugin '{name}' load errors: " + "; ".join(errors))
        else:
            app.logger.info(f"Plugin '{name}' loaded successfully.")

    @app.context_processor
    def inject_current_year():
        return {'current_year': datetime.now().year}

    @app.context_processor
    def inject_now():
        return {'utcnow': datetime.utcnow()}

    @app.before_request
    def start_timer():
        request._start_time = time.time()

    @app.after_request
    def log_analytics(response):
        from plugins.admin.models import AnalyticsEvent  # noqa: E402
        try:
            dur = time.time() - getattr(request, '_start_time', time.time())
            ev = AnalyticsEvent(
                method=request.method,
                path=request.path,
                status_code=response.status_code,
                response_time=dur,
                user_agent=request.headers.get('User-Agent'),
                referer=request.headers.get('Referer'),
                accept_language=request.headers.get('Accept-Language'),
            )
            db.session.add(ev)
            db.session.commit()
        except Exception:
            db.session.rollback()
        return response

    @app.teardown_appcontext
    def shutdown_session(exception=None):
        db.session.remove()

    from plugins.media.routes import generate_image_url  # noqa: E402
    app.jinja_env.globals['generate_image_url'] = generate_image_url

    return app
