# File: app/__init__.py

import os
import json
import importlib
import time
import logging
from datetime import datetime

from dotenv import load_dotenv, find_dotenv
from flask import Flask, request
from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate, upgrade, migrate as _migrate, stamp as _stamp
from flask_login import LoginManager
from flask_wtf.csrf import CSRFProtect

# ─── Core extensions ───────────────────────────────────────────────────────────
db            = SQLAlchemy()
migrate       = Migrate()
login_manager = LoginManager()
csrf          = CSRFProtect()


def create_app():
    # ─── Load .env ───────────────────────────────────────────────────────────────
    dotenv_path = find_dotenv()
    if dotenv_path:
        load_dotenv(dotenv_path, override=True)

    # ─── Flask setup ─────────────────────────────────────────────────────────────
    project_root   = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    migrations_dir = os.path.join(project_root, 'migrations')

    app = Flask(
        __name__,
        static_folder=os.path.join(project_root, 'static'),
        static_url_path='/static'
    )

    # ←— install INFO‐level logging handler
    handler = logging.StreamHandler()
    handler.setLevel(logging.INFO)
    app.logger.setLevel(logging.INFO)
    app.logger.addHandler(handler)

    app.logger.info("🚀  Starting create_app()")

    # ←— main config
    app.config.from_object('app.config.Config')
    app.logger.info(f"🔧  Loaded config from {app.config.__class__.__name__}")

    # ─── Init extensions ─────────────────────────────────────────────────────────
    csrf.init_app(app)
    db.init_app(app)
    migrate.init_app(app, db, directory=migrations_dir)
    login_manager.init_app(app)
    login_manager.login_view = 'auth.login'
    app.logger.info("🔗  Initialized extensions (CSRF, SQLAlchemy, Migrate, LoginManager)")

    # ─── AUTOMATIC MIGRATIONS ────────────────────────────────────────────────────
    with app.app_context():
        app.logger.info("🛠️  Checking for schema changes…")

        # 1) Ensure version table exists and stamp if needed
        try:
            upgrade()
            app.logger.info("🛠️  Alembic reports DB is at head")
        except Exception as e:
            app.logger.debug(f"DB not at head, stamping to current head: {e}")
            try:
                stamp = _stamp  # flask_migrate.stamp
                stamp(revision='head')
                app.logger.info("🛠️  Stamped database to head")
            except Exception as e2:
                app.logger.error(f"Stamp to head failed: {e2}")
            upgrade()
            app.logger.info("🛠️  Upgraded DB after stamping")

        # 2) Autogenerate a new migration if there are any model/schema diffs
        try:
            _migrate(message="autogen", autogenerate=True)
            app.logger.info("🛠️  Autogenerated migration revision created")
        except Exception as e:
            app.logger.debug(f"No new migrations detected: {e}")

        # 3) Apply all migrations up to head
        try:
            upgrade()
            app.logger.info("🛠️  Database upgraded to head")
        except Exception as e:
            app.logger.error(f"Failed to apply migrations: {e}")
            raise

    # ─── Core routes & error‐handlers ─────────────────────────────────────────────
    from .errors import bp as errors_bp  # noqa: E402
    app.register_blueprint(errors_bp)
    app.logger.info("✔️  Registered error‐handler blueprint")

    from .routes import init_app as register_core_routes  # noqa: E402
    register_core_routes(app)
    app.logger.info("✔️  Registered core routes")

    # ─── JSON‐driven plugin loader ────────────────────────────────────────────────
    app.logger.info("🔌  Discovering plugins…")
    plugins_dir = os.path.join(project_root, 'plugins')
    for name in sorted(os.listdir(plugins_dir)):
        manifest = os.path.join(plugins_dir, name, 'plugin.json')
        if not os.path.isfile(manifest):
            continue

        errors = []
        try:
            meta = json.load(open(manifest))
        except Exception as e:
            app.logger.error(f"Plugin '{name}' manifest load error: {e}")
            continue

        # 1) models
        for model_path in meta.get('models', []):
            try:
                importlib.import_module(model_path)
            except Exception as e:
                errors.append(f"model '{model_path}': {e}")

        # 2) user_loader
        ul_cfg = meta.get('user_loader')
        if ul_cfg:
            try:
                ul_mod = importlib.import_module(ul_cfg['module'])
                ul_fn  = getattr(ul_mod, ul_cfg['callable'])
                ul_fn(login_manager)
            except Exception as e:
                errors.append(f"user_loader '{ul_cfg['module']}:{ul_cfg['callable']}': {e}")

        # 3) routes
        routes_cfg = meta.get('routes')
        if routes_cfg:
            try:
                mod    = importlib.import_module(routes_cfg['module'])
                bp_obj = getattr(mod, routes_cfg['blueprint'])
                prefix = routes_cfg.get('url_prefix')
                app.register_blueprint(bp_obj, url_prefix=prefix, strict_slashes=False)
            except Exception as e:
                errors.append(f"routes '{routes_cfg['module']}': {e}")

        # 4) CLI
        cli_cfg = meta.get('cli')
        if cli_cfg:
            try:
                mod = importlib.import_module(cli_cfg['module'])
                fn  = getattr(mod, cli_cfg['callable'])
                app.cli.add_command(fn)
            except Exception as e:
                errors.append(f"cli '{cli_cfg['module']}:{cli_cfg['callable']}': {e}")

        # 5) template_globals
        for tg in meta.get('template_globals', []):
            try:
                mod_name, fn_name = tg['callable'].rsplit('.', 1)
                mod = importlib.import_module(mod_name)
                fn  = getattr(mod, fn_name)
                app.jinja_env.globals[tg['name']] = fn
            except Exception as e:
                errors.append(f"template_global '{tg}': {e}")

        # 6) subplugins
        for sp in meta.get('subplugins', []):
            for mp in sp.get('models', []):
                try:
                    importlib.import_module(mp)
                except Exception as e:
                    errors.append(f"subplugin model '{mp}': {e}")
            sp_rt = sp.get('routes')
            if sp_rt:
                try:
                    mod    = importlib.import_module(sp_rt['module'])
                    bp_obj = getattr(mod, sp_rt['blueprint'])
                    prefix = sp_rt.get('url_prefix')
                    app.register_blueprint(bp_obj, url_prefix=prefix, strict_slashes=False)
                except Exception as e:
                    errors.append(f"subplugin routes '{sp_rt['module']}': {e}")

        if errors:
            app.logger.error(f"Plugin '{name}' failed to load: " + "; ".join(errors))
        else:
            app.logger.info(f"Plugin '{name}' loaded successfully.")

    # ─── Context processors, analytics & teardown ───────────────────────────────
    @app.context_processor
    def inject_current_year():
        return {'current_year': datetime.now().year}

    @app.context_processor
    def inject_now():
        return {'utcnow': datetime.utcnow()}

    @app.before_request
    def start_timer():
        request._start_time = time.time()

    @app.after_request
    def log_analytics(response):
        from plugins.admin.models import AnalyticsEvent  # noqa: E402
        try:
            duration = time.time() - getattr(request, '_start_time', time.time())
            ev = AnalyticsEvent(
                method          = request.method,
                path            = request.path,
                status_code     = response.status_code,
                response_time   = duration,
                user_agent      = request.headers.get('User-Agent'),
                referer         = request.headers.get('Referer'),
                accept_language = request.headers.get('Accept-Language'),
            )
            db.session.add(ev)
            db.session.commit()
        except Exception:
            db.session.rollback()
        return response

    @app.teardown_appcontext
    def shutdown_session(exception=None):
        db.session.remove()

    # ─── generate_image_url helper (avoids circular import) ─────────────────────
    from plugins.media.routes import generate_image_url  # noqa: E402
    app.jinja_env.globals['generate_image_url'] = generate_image_url

    app.logger.info("✅  create_app() complete; ready to serve")
    return app
