# File: app/__init__.py

import os
import json
import importlib
import time
import logging
from datetime import datetime

from dotenv import load_dotenv, find_dotenv
from flask import Flask, request
from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate, upgrade, migrate as _migrate, stamp as _stamp
from flask_login import LoginManager
from flask_wtf.csrf import CSRFProtect

# ─── Core extensions ───────────────────────────────────────────────────────────
db            = SQLAlchemy()
migrate       = Migrate()
login_manager = LoginManager()
csrf          = CSRFProtect()


def create_app():
    # ─── Load .env ───────────────────────────────────────────────────────────────
    dotenv_path = find_dotenv()
    if dotenv_path:
        load_dotenv(dotenv_path, override=True)

    # ─── Flask setup ─────────────────────────────────────────────────────────────
    project_root   = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    migrations_dir = os.path.join(project_root, 'migrations')

    app = Flask(
        __name__,
        static_folder=os.path.join(project_root, 'static'),
        static_url_path='/static'
    )

    # install INFO‐level logging handler
    handler = logging.StreamHandler()
    handler.setLevel(logging.INFO)
    app.logger.setLevel(logging.INFO)
    app.logger.addHandler(handler)
    app.logger.info("🚀 Starting create_app()")

    # main config
    app.config.from_object('app.config.Config')
    app.logger.info(f"🔧 Loaded config from {app.config.__class__.__name__}")

    # ─── Init extensions ─────────────────────────────────────────────────────────
    csrf.init_app(app)
    db.init_app(app)
    migrate.init_app(app, db, directory=migrations_dir)
    login_manager.init_app(app)
    login_manager.login_view = 'auth.login'
    app.logger.info("🔗 Initialized extensions (CSRF, SQLAlchemy, Migrate, LoginManager)")

    # ─── AUTOMATIC MIGRATIONS ────────────────────────────────────────────────────
    with app.app_context():
        app.logger.info("🛠️ Checking for schema changes…")
        try:
            upgrade()
            app.logger.info("🛠️ Alembic reports DB is at head")
        except Exception:
            _stamp(revision='head')
            upgrade()
            app.logger.info("🛠️ Stamped and upgraded to head")
        try:
            _migrate(message="autogen", autogenerate=True)
            app.logger.info("🛠️ Autogenerated migration revision created")
        except Exception:
            app.logger.debug("🛠️ No new migrations detected")
        upgrade()
        app.logger.info("🛠️ Database fully migrated")

    # ─── Core routes & error‐handlers ─────────────────────────────────────────────
    from .errors import bp as errors_bp  # noqa: E402
    app.register_blueprint(errors_bp)
    from .routes import init_app as register_core_routes  # noqa: E402
    register_core_routes(app)

    # ─── JSON‐driven plugin loader with summary ───────────────────────────────────
    app.logger.info("🔌 Discovering plugins…")
    plugins_dir    = os.path.join(project_root, 'plugins')
    loaded_plugins = []

    for name in sorted(os.listdir(plugins_dir)):
        manifest = os.path.join(plugins_dir, name, 'plugin.json')
        if not os.path.isfile(manifest):
            continue

        plugin_info = {
            'name': name,
            'models': [],
            'routes': None,
            'cli': [],
            'template_globals': [],
            'tasks': [],
            'tasks_init': [],
            'subplugins': []
        }
        errors = []

        try:
            meta = json.load(open(manifest))
        except Exception as e:
            app.logger.error(f"Plugin '{name}' manifest load error: {e}")
            continue

        # 1) models
        for model_path in meta.get('models', []):
            try:
                importlib.import_module(model_path)
                plugin_info['models'].append(model_path)
            except Exception as e:
                errors.append(f"model '{model_path}': {e}")

        # 2) user_loader
        ul_cfg = meta.get('user_loader')
        if ul_cfg:
            try:
                ul_mod = importlib.import_module(ul_cfg['module'])
                ul_fn  = getattr(ul_mod, ul_cfg['callable'])
                ul_fn(login_manager)
            except Exception as e:
                errors.append(f"user_loader '{ul_cfg['module']}:{ul_cfg['callable']}': {e}")

        # 3) routes
        routes_cfg = meta.get('routes')
        if routes_cfg:
            try:
                mod    = importlib.import_module(routes_cfg['module'])
                bp_obj = getattr(mod, routes_cfg['blueprint'])
                prefix = routes_cfg.get('url_prefix')
                app.register_blueprint(bp_obj, url_prefix=prefix, strict_slashes=False)
                plugin_info['routes'] = f"{routes_cfg['module']} ({routes_cfg['blueprint']})"
            except Exception as e:
                errors.append(f"routes '{routes_cfg['module']}': {e}")

        # 4) CLI
        cli_cfg = meta.get('cli')
        if cli_cfg:
            try:
                mod = importlib.import_module(cli_cfg['module'])
                fn  = getattr(mod, cli_cfg['callable'])
                app.cli.add_command(fn)
                plugin_info['cli'].append(f"{cli_cfg['module']}:{cli_cfg['callable']}")
            except Exception as e:
                errors.append(f"cli '{cli_cfg['module']}:{cli_cfg['callable']}': {e}")

        # 5) template_globals
        for tg in meta.get('template_globals', []):
            try:
                mod_name, fn_name = tg['callable'].rsplit('.', 1)
                mod = importlib.import_module(mod_name)
                fn  = getattr(mod, fn_name)
                app.jinja_env.globals[tg['name']] = fn
                plugin_info['template_globals'].append(tg['name'])
            except Exception as e:
                errors.append(f"template_global '{tg}': {e}")

        # 6) subplugins
        for sp in meta.get('subplugins', []):
            sub_info = {'name': sp['name'], 'models': [], 'routes': None}
            for mp in sp.get('models', []):
                try:
                    importlib.import_module(mp)
                    sub_info['models'].append(mp)
                except Exception as e:
                    errors.append(f"subplugin model '{mp}': {e}")
            sp_rt = sp.get('routes')
            if sp_rt:
                try:
                    mod    = importlib.import_module(sp_rt['module'])
                    bp_obj = getattr(mod, sp_rt['blueprint'])
                    prefix = sp_rt.get('url_prefix')
                    app.register_blueprint(bp_obj, url_prefix=prefix, strict_slashes=False)
                    sub_info['routes'] = f"{sp_rt['module']} ({sp_rt['blueprint']})"
                except Exception as e:
                    errors.append(f"subplugin routes '{sp_rt['module']}': {e}")
            plugin_info['subplugins'].append(sub_info)

        # 7) tasks
        for task_mod in meta.get('tasks', []):
            try:
                importlib.import_module(task_mod)
                plugin_info['tasks'].append(task_mod)
            except Exception as e:
                errors.append(f"task '{task_mod}': {e}")

        # 8) tasks_init
        for hook in meta.get('tasks_init', []):
            try:
                init_mod = importlib.import_module(hook['module'])
                init_fn  = getattr(init_mod, hook['callable'])
                init_fn(app)
                plugin_info['tasks_init'].append(f"{hook['module']}:{hook['callable']}")
            except Exception as e:
                errors.append(f"tasks_init '{hook['module']}:{hook['callable']}': {e}")

        if errors:
            app.logger.error(f"Plugin '{name}' failed to load: " + "; ".join(errors))
        else:
            app.logger.info(f"Plugin '{name}' loaded successfully.")
            loaded_plugins.append(plugin_info)

    # ─── Log summary of loaded plugins ────────────────────────────────────────────
    app.logger.info("🌟 Loaded plugins summary:")
    for info in loaded_plugins:
        app.logger.info(
            f" • {info['name']}: models={info['models']}, "
            f"routes={info['routes']}, cli={info['cli']}, "
            f"template_globals={info['template_globals']}, "
            f"tasks={info['tasks']}, tasks_init={info['tasks_init']}, "
            f"subplugins={[sub['name'] for sub in info['subplugins']]}"
        )

    # ─── Context processors, before/after request, teardown ─────────────────────
    @app.context_processor
    def inject_current_year():
        return {'current_year': datetime.now().year}

    @app.context_processor
    def inject_now():
        return {'utcnow': datetime.utcnow()}

    @app.before_request
    def start_timer():
        request._start_time = time.time()

    @app.after_request
    def log_analytics(response):
        from plugins.admin.models import AnalyticsEvent  # noqa: E402
        try:
            duration = time.time() - getattr(request, '_start_time', time.time())
            ev = AnalyticsEvent(
                method          = request.method,
                path            = request.path,
                status_code     = response.status_code,
                response_time   = duration,
                user_agent      = request.headers.get('User-Agent'),
                referer         = request.headers.get('Referer'),
                accept_language = request.headers.get('Accept-Language'),
            )
            db.session.add(ev)
            db.session.commit()
        except Exception:
            db.session.rollback()
        return response

    @app.teardown_appcontext
    def shutdown_session(exception=None):
        db.session.remove()

    app.logger.info("✅ create_app() complete; ready to serve")
    return app
