# app/__init__.py

import os
import json
import time
import glob
import importlib
import importlib.util

from flask import Flask, request
from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate
from flask_login import LoginManager
from flask_wtf.csrf import CSRFProtect
from dotenv import load_dotenv, find_dotenv
from datetime import datetime

# ─── Load .env into os.environ (wherever it lives) ────────────────────────────
dotenv_path = find_dotenv()
if dotenv_path:
    load_dotenv(dotenv_path, override=True)

# ─── Initialize core extensions ────────────────────────────────────────────────
db            = SQLAlchemy()
migrate       = Migrate()
login_manager = LoginManager()
csrf          = CSRFProtect()

# We need this in templates, so import it now…
from plugins.media.routes import generate_image_url  # noqa: E402

def create_app():
    app = Flask(__name__)
    app.config.from_object('app.config.Config')

    # ─── Initialize extensions with the app ────────────────────────────────────
    csrf.init_app(app)
    db.init_app(app)
    migrate.init_app(app, db)
    login_manager.init_app(app)
    login_manager.login_view = 'auth.login'

    # ─── Flask-Login user loader ───────────────────────────────────────────────
    from plugins.auth.models import User  # noqa: E402

    @login_manager.user_loader
    def load_user(user_id):
        try:
            return User.query.get(int(user_id))
        except Exception:
            return None

    # ─── Error handlers ────────────────────────────────────────────────────────
    from .errors import bp as errors_bp  # noqa: E402
    app.register_blueprint(errors_bp)

    # ─── 1) Auto-import **all** plugin models recursively ─────────────────────────
    root_plugins = os.path.abspath(
        os.path.join(os.path.dirname(__file__), '..', 'plugins')
    )
    for model_path in glob.glob(
        os.path.join(root_plugins, '**', 'models.py'),
        recursive=True
    ):
        # e.g. /app/plugins/plant/growlog/models.py → plugins.plant.growlog.models
        rel = os.path.relpath(model_path, root_plugins).replace(os.sep, '.')
        pkg = f"plugins.{rel[:-3]}"  # strip trailing '.py'
        try:
            importlib.import_module(pkg)
            print(f"✅ (Startup) Loaded: {pkg}")
        except Exception as e:
            print(f"❌ (Startup) Failed to load {pkg}: {e}")

    # ─── 2) Auto-discover & register routes, CLI and entry-points for **all** plugins
    for root, dirs, files in os.walk(root_plugins):
        # skip any directory marked “.noload”
        if root.endswith('.noload'):
            continue

        rel = os.path.relpath(root, root_plugins)
        if rel == '.':  # skip the top‐level “plugins” folder
            continue

        plugin_name = rel.replace(os.sep, '.')
        pkg_base    = f"plugins.{plugin_name}"

        # (a) routes.py
        if 'routes.py' in files:
            route_file = os.path.join(root, 'routes.py')
            try:
                spec = importlib.util.spec_from_file_location(
                    f"{pkg_base}.routes", route_file
                )
                mod  = importlib.util.module_from_spec(spec)
                spec.loader.exec_module(mod)
                if hasattr(mod, 'bp'):
                    app.register_blueprint(mod.bp, strict_slashes=False)
                    print(f"✔️ Registered routes for plugin '{plugin_name}'")
            except Exception as e:
                print(f"❌ Failed to load routes from plugin '{plugin_name}': {e}")

        # (b) __init__.py → register_cli & entry_point
        if '__init__.py' in files:
            try:
                cli_mod = importlib.import_module(pkg_base)
                if hasattr(cli_mod, 'register_cli'):
                    cli_mod.register_cli(app)
                    print(f"✔️ Registered CLI for plugin '{plugin_name}'")

                plugin_json = os.path.join(root, 'plugin.json')
                if os.path.isfile(plugin_json):
                    with open(plugin_json, 'r') as f:
                        meta = json.load(f)
                    entry = meta.get('entry_point')
                    if entry and hasattr(cli_mod, entry):
                        getattr(cli_mod, entry)(app)
                        print(f"✔️ Ran entry point '{entry}' for plugin '{plugin_name}'")
            except Exception as e:
                print(f"❌ Failed to load CLI for plugin '{plugin_name}': {e}")

    # ─── Inject convenience globals into Jinja ─────────────────────────────────
    @app.context_processor
    def inject_current_year():
        return {'current_year': datetime.now().year}

    @app.context_processor
    def inject_now():
        return {'utcnow': datetime.utcnow()}

    # ─── Simple timing / analytics hooks ────────────────────────────────────────
    @app.before_request
    def start_timer():
        request._start_time = time.time()

    @app.after_request
    def log_analytics(response):
        from plugins.admin.models import AnalyticsEvent  # noqa: E402
        try:
            duration = time.time() - getattr(request, '_start_time', time.time())
            ev = AnalyticsEvent(
                method=request.method,
                path=request.path,
                status_code=response.status_code,
                response_time=duration,
                user_agent=request.headers.get('User-Agent'),
                referer=request.headers.get('Referer'),
                accept_language=request.headers.get('Accept-Language'),
            )
            db.session.add(ev)
            db.session.commit()
        except Exception:
            db.session.rollback()
        return response

    # ─── Cleanup our session on teardown ───────────────────────────────────────
    @app.teardown_appcontext
    def shutdown_session(exception=None):
        db.session.remove()

    # ─── Make generate_image_url available in every template ────────────────────
    app.jinja_env.globals['generate_image_url'] = generate_image_url

    return app
