# File: app/__init__.py

import os
import json
import time
import glob
import importlib

from flask import Flask, request
from flask_sqlalchemy import SQLAlchemy
from flask_migrate import Migrate
from flask_login import LoginManager
from flask_wtf.csrf import CSRFProtect
from datetime import datetime

# ─── Load .env from any parent ─────────────────────────────────────────────────
from dotenv import load_dotenv, find_dotenv
dotenv_path = find_dotenv()
if dotenv_path:
    load_dotenv(dotenv_path, override=True)

# ─── Core extensions ───────────────────────────────────────────────────────────
db            = SQLAlchemy()
migrate       = Migrate()
login_manager = LoginManager()
csrf          = CSRFProtect()

# Make generate_image_url available immediately in templates
from plugins.media.routes import generate_image_url  # noqa: E402

def create_app():
    # ─── Configure Flask ────────────────────────────────────────────────────────
    project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    app = Flask(
        __name__,
        static_folder=os.path.join(project_root, 'static'),
        static_url_path='/static'
    )
    app.config.from_object('app.config.Config')

    # ─── Initialize extensions ─────────────────────────────────────────────────
    csrf.init_app(app)
    db.init_app(app)
    migrate.init_app(app, db)
    login_manager.init_app(app)
    login_manager.login_view = 'auth.login'

    # ─── Flask-Login user loader ───────────────────────────────────────────────
    from plugins.auth.models import User  # noqa: E402
    @login_manager.user_loader
    def load_user(user_id):
        return User.query.get(int(user_id)) if user_id.isdigit() else None

    # ─── Register error handlers ────────────────────────────────────────────────
    from .errors import bp as errors_bp  # noqa: E402
    app.register_blueprint(errors_bp)

    # ─── Register core `/` and `/health` endpoints ─────────────────────────────
    from .routes import init_app as register_core_routes  # noqa: E402
    register_core_routes(app)
    print("✔️ Registered core routes")

    # ─── Auto‐import all plugin models ──────────────────────────────────────────
    root_plugins = os.path.join(project_root, 'plugins')
    for model_path in glob.glob(os.path.join(root_plugins, '**', 'models.py'), recursive=True):
        rel = os.path.relpath(model_path, root_plugins).replace(os.sep, '.')
        pkg = f"plugins.{rel[:-3]}"  # strip .py
        try:
            importlib.import_module(pkg)
            print(f"✅ Loaded models: {pkg}")
        except Exception as e:
            print(f"❌ Error loading models from {pkg}: {e}")

    # ─── Auto‐discover & register each plugin’s routes & CLI ────────────────────
    for root, dirs, files in os.walk(root_plugins):
        rel = os.path.relpath(root, root_plugins)
        if rel == '.' or root.endswith('.noload'):
            continue

        plugin_pkg = f"plugins.{rel.replace(os.sep, '.')}"
        # (a) routes.py
        if 'routes.py' in files:
            try:
                mod = importlib.import_module(f"{plugin_pkg}.routes")
                if hasattr(mod, 'bp'):
                    app.register_blueprint(mod.bp, strict_slashes=False)
                    print(f"✔️ Registered routes: {plugin_pkg}.routes")
            except Exception as e:
                print(f"❌ Error registering routes for {plugin_pkg}: {e}")

        # (b) __init__.py for CLI & entry_point
        if '__init__.py' in files:
            try:
                cli_mod = importlib.import_module(plugin_pkg)
                if hasattr(cli_mod, 'register_cli'):
                    cli_mod.register_cli(app)
                    print(f"✔️ Registered CLI: {plugin_pkg}")
                plugin_json = os.path.join(root, 'plugin.json')
                if os.path.isfile(plugin_json):
                    meta = json.load(open(plugin_json))
                    entry = meta.get('entry_point')
                    if entry and hasattr(cli_mod, entry):
                        getattr(cli_mod, entry)(app)
                        print(f"✔️ Ran entry_point '{entry}': {plugin_pkg}")
            except Exception as e:
                print(f"❌ Error loading CLI for {plugin_pkg}: {e}")

    # ─── Jinja context processors ───────────────────────────────────────────────
    @app.context_processor
    def inject_current_year():
        return {'current_year': datetime.now().year}

    @app.context_processor
    def inject_now():
        return {'utcnow': datetime.utcnow()}

    # ─── Simple timing / analytics hooks ────────────────────────────────────────
    @app.before_request
    def start_timer():
        request._start_time = time.time()

    @app.after_request
    def log_analytics(response):
        from plugins.admin.models import AnalyticsEvent  # noqa: E402
        try:
            duration = time.time() - getattr(request, '_start_time', time.time())
            ev = AnalyticsEvent(
                method         = request.method,
                path           = request.path,
                status_code    = response.status_code,
                response_time  = duration,
                user_agent     = request.headers.get('User-Agent'),
                referer        = request.headers.get('Referer'),
                accept_language=request.headers.get('Accept-Language'),
            )
            db.session.add(ev)
            db.session.commit()
        except Exception:
            db.session.rollback()
        return response

    # ─── Cleanup DB session on teardown ────────────────────────────────────────
    @app.teardown_appcontext
    def shutdown_session(exception=None):
        db.session.remove()

    # ─── Make generate_image_url available in every template ────────────────────
    app.jinja_env.globals['generate_image_url'] = generate_image_url

    return app
