# File: app/celery_app.py

import os
import json
import importlib
from celery import Celery

# create celery instance
celery = Celery(
    'natureinpots',
    broker=os.getenv('CELERY_BROKER_URL', None),
    backend=os.getenv('CELERY_RESULT_BACKEND', None),
)


def init_celery(app):
    """
    Initialize celery by wiring it up to the Flask app context,
    loading any plugin‐defined tasks and schedules.
    """
    # 1) Broker & backend from Flask config
    celery.conf.broker_url    = app.config['CELERY_BROKER_URL']
    celery.conf.result_backend = app.config['CELERY_RESULT_BACKEND']

    # 2) ensure tasks run inside Flask app context
    TaskBase = celery.Task

    class ContextTask(TaskBase):
        def __call__(self, *args, **kwargs):
            with app.app_context():
                return TaskBase.__call__(self, *args, **kwargs)

    celery.Task = ContextTask

    # 3) discover plugins and pull in their task modules & init hooks
    plugins_dir  = os.path.join(app.root_path, '..', 'plugins')
    task_modules = []

    for plugin_name in sorted(os.listdir(plugins_dir)):
        manifest = os.path.join(plugins_dir, plugin_name, 'plugin.json')
        if not os.path.isfile(manifest):
            continue

        try:
            meta = json.load(open(manifest, 'r'))
        except Exception:
            app.logger.error(f"[Celery] Failed loading plugin.json for '{plugin_name}'")
            continue

        # a) collect anything in "tasks" for autodiscovery
        tasks_cfg = meta.get('tasks')
        if isinstance(tasks_cfg, str):
            task_modules.append(tasks_cfg)
        elif isinstance(tasks_cfg, dict) and tasks_cfg.get('module'):
            task_modules.append(tasks_cfg['module'])
        elif isinstance(tasks_cfg, list):
            for entry in tasks_cfg:
                if isinstance(entry, str):
                    task_modules.append(entry)
                elif isinstance(entry, dict) and entry.get('module'):
                    task_modules.append(entry['module'])

        # b) fire any tasks_init hooks (e.g. set up beat schedules)
        for hook in meta.get('tasks_init', []):
            try:
                m  = importlib.import_module(hook['module'])
                fn = getattr(m, hook['callable'])
                # ← pass the Celery instance, not the Flask app
                fn(celery)
            except Exception as e:
                app.logger.error(f"[Celery] Failed tasks_init for '{plugin_name}': {e}")

    # 4) Autodiscover all the tasks in those modules
    if task_modules:
        celery.autodiscover_tasks(task_modules)
        app.logger.info(f"[Celery] Autodiscovered tasks in: {task_modules}")

    return celery
