# File: app/celery_app.py

import os
import json
import importlib

from celery import Celery

# ─── Create a module-level Celery instance ──────────────────────────────────
celery = Celery(
    'natureinpots',
    task_serializer='json',
    accept_content=['json']
)


def init_celery(app):
    """
    Configure module-level `celery` from Flask config, wrap tasks in app context,
    autodiscover plugin task modules, and init any plugin tasks_init hooks.
    """
    # 1) Broker & backend
    celery.conf.broker_url       = app.config['CELERY_BROKER_URL']
    celery.conf.result_backend   = app.config['CELERY_RESULT_BACKEND']

    # 2) Ensure tasks run inside Flask context
    class ContextTask(celery.Task):
        def __call__(self, *args, **kwargs):
            with app.app_context():
                return self.run(*args, **kwargs)
    celery.Task = ContextTask

    # 3) Scan all plugins for both "tasks" modules and "tasks_init" hooks
    project_root = os.path.abspath(os.path.join(app.root_path, '..'))
    plugins_dir  = os.path.join(project_root, 'plugins')

    task_modules = []
    for plugin_name in sorted(os.listdir(plugins_dir)):
        manifest = os.path.join(plugins_dir, plugin_name, 'plugin.json')
        if not os.path.isfile(manifest):
            continue

        try:
            meta = json.load(open(manifest))
        except Exception:
            continue

        # 3a) Collect the "tasks" module if declared
        tasks_cfg = meta.get('tasks')
        if isinstance(tasks_cfg, dict) and tasks_cfg.get('module'):
            task_modules.append(tasks_cfg['module'])

        # 3b) Run any "tasks_init" hooks
        for hook in meta.get('tasks_init', []):
            try:
                m  = importlib.import_module(hook['module'])
                fn = getattr(m, hook['callable'])
                fn(app)
            except Exception as e:
                app.logger.error(f"[Celery] tasks_init for '{plugin_name}' failed: {e}")

    # 4) Tell Celery to autodiscover all collected task modules
    if task_modules:
        celery.autodiscover_tasks(task_modules)
        app.logger.info(f"[Celery] Autodiscovered tasks in: {task_modules}")

    return celery


# ─── Auto-initialize at import time ────────────────────────────────────────────
# so that "celery -A app.celery_app:celery" always knows about your tasks.
from app import create_app
_flask_app = create_app()
init_celery(_flask_app)
