# File: app/celery_app.py

import os
import json
import importlib
from celery import Celery

# ─── Create a module‐level Celery instance ──────────────────────────────────
celery = Celery('natureinpots', task_serializer='json')

def init_celery(app):
    """
    Configure our module‐level `celery` with Flask settings,
    wrap tasks in app context, and init any plugin tasks hooks.
    """
    # 1) Load broker & backend from Flask config
    celery.conf.broker_url    = app.config['CELERY_BROKER_URL']
    celery.conf.result_backend = app.config.get('CELERY_RESULT_BACKEND')

    # 2) Ensure tasks run inside Flask context
    class ContextTask(celery.Task):
        def __call__(self, *args, **kwargs):
            with app.app_context():
                return self.run(*args, **kwargs)
    celery.Task = ContextTask

    # 3) Discover & initialize plugin tasks from plugin.json
    project_root = os.path.abspath(os.path.join(app.root_path, '..'))
    plugins_dir  = os.path.join(project_root, 'plugins')
    for name in sorted(os.listdir(plugins_dir)):
        manifest = os.path.join(plugins_dir, name, 'plugin.json')
        if not os.path.isfile(manifest):
            continue
        try:
            meta = json.load(open(manifest))
        except Exception:
            continue
        tasks_cfg = meta.get('tasks')
        if tasks_cfg:
            try:
                mod = importlib.import_module(tasks_cfg['module'])
                fn  = getattr(mod, tasks_cfg['callable'])
                fn(app)  # e.g. schedule periodic jobs
            except Exception as e:
                app.logger.error(f"[Celery] Failed to init tasks for '{name}': {e}")

    return celery
