# File: app/plugin_loader.py

import os
import json
import importlib

def register_plugins(app):
    project_root = os.path.abspath(os.path.join(os.path.dirname(__file__), '..'))
    plugins_dir  = os.path.join(project_root, 'plugins')

    app.logger.info("🔌 Discovering plugins…")
    for name in sorted(os.listdir(plugins_dir)):
        manifest = os.path.join(plugins_dir, name, 'plugin.json')
        if not os.path.isfile(manifest):
            continue

        try:
            meta = json.load(open(manifest))
        except Exception as e:
            app.logger.error(f"[Plugins] Failed to load {name}/plugin.json: {e}")
            continue

        # 1) import models
        for model_path in meta.get('models', []):
            importlib.import_module(model_path)

        # 2) register routes
        rt = meta.get('routes')
        if rt:
            m      = importlib.import_module(rt['module'])
            bp_obj = getattr(m, rt['blueprint'])
            app.register_blueprint(
                bp_obj,
                url_prefix=rt.get('url_prefix'),
                strict_slashes=False
            )

        # 3) register CLI commands
        for cli in meta.get('cli', []):
            m  = importlib.import_module(cli['module'])
            fn = getattr(m, cli['callable'])
            app.cli.add_command(fn)

        # 4) template globals
        for tg in meta.get('template_globals', []):
            mod_name, fn_name = tg['callable'].rsplit('.', 1)
            m  = importlib.import_module(mod_name)
            fn = getattr(m, fn_name)
            app.jinja_env.globals[tg['name']] = fn

        # 5) subplugins (just models & routes)
        for sp in meta.get('subplugins', []):
            for mp in sp.get('models', []):
                importlib.import_module(mp)
            srt = sp.get('routes')
            if srt:
                m      = importlib.import_module(srt['module'])
                bp_obj = getattr(m, srt['blueprint'])
                app.register_blueprint(
                    bp_obj,
                    url_prefix=srt.get('url_prefix'),
                    strict_slashes=False
                )

        app.logger.info(f"[Plugins] Loaded '{name}'")
