from uuid import UUID as _UUID
from werkzeug.exceptions import NotFound
from flask import (
    Blueprint, render_template, abort, redirect, url_for, request, flash
)
from flask_login import login_required, current_user
from app import db
from .models import GrowLog
from .forms import GrowLogForm
from plugins.plant.models import Plant, PlantCommonName

bp = Blueprint(
    'growlog',
    __name__,
    url_prefix='/growlogs',
)

def _get_plant_by_uuid(plant_uuid):
    try:
        pu = _UUID(plant_uuid)
    except Exception:
        abort(404)
    plant = Plant.query.filter_by(uuid=pu).first()
    if not plant:
        raise NotFound()
    return plant

@bp.route('/', methods=['GET'])
@login_required
def list_all_logs():
    logs = GrowLog.query.order_by(GrowLog.created_at.desc()).all()
    return render_template('growlog/log_list.html', logs=logs, plant=None)

@bp.route('/<uuid:plant_uuid>/', methods=['GET'])
@login_required
def list_logs(plant_uuid):
    plant = _get_plant_by_uuid(plant_uuid)
    logs = GrowLog.query.filter_by(plant_id=plant.id).order_by(GrowLog.created_at.desc()).all()
    return render_template('growlog/log_list.html', logs=logs, plant=plant)

@bp.route('/add', methods=['GET', 'POST'])
@bp.route('/<uuid:plant_uuid>/add', methods=['GET', 'POST'])
@login_required
def add_log(plant_uuid=None):
    # prepare form
    form = GrowLogForm()
    plants = Plant.query.all()
    form.plant_uuid.choices = [(str(p.uuid), p.common_name.name) for p in plants]

    plant = None
    hide_select = False
    if plant_uuid:
        plant = _get_plant_by_uuid(plant_uuid)
        form.plant_uuid.data = str(plant.uuid)
        hide_select = True

    if form.validate_on_submit():
        pu = _UUID(form.plant_uuid.data)
        plant = Plant.query.filter_by(uuid=pu).first_or_404()
        log = GrowLog(
            plant_id=plant.id,
            event_type=form.event_type.data,
            title=form.title.data,
            notes=form.notes.data,
            is_public=form.is_public.data
        )
        db.session.add(log)
        db.session.commit()
        flash('Grow log added.', 'success')
        return redirect(url_for('growlog.list_logs', plant_uuid=plant_uuid or form.plant_uuid.data))

    return render_template(
        'growlog/log_form.html',
        form=form,
        plant=plant,
        hide_plant_select=hide_select
    )

@bp.route('/<uuid:plant_uuid>/delete/<int:log_id>', methods=['POST'])
@login_required
def delete_log(plant_uuid, log_id):
    plant = _get_plant_by_uuid(plant_uuid)
    log = GrowLog.query.filter_by(id=log_id, plant_id=plant.id).first_or_404()
    db.session.delete(log)
    db.session.commit()
    flash('Grow log deleted.', 'warning')
    return redirect(url_for('growlog.list_logs', plant_uuid=plant_uuid))
