# plugins/utility/tasks.py

import csv
import os
import zipfile
import tempfile
import shutil
import io
from datetime import datetime

from celery import current_app as celery_app
from flask import current_app
from app import db
from app.neo4j_utils import get_neo4j_handler
from plugins.plant.models import (
    Plant, PlantCommonName, PlantScientificName, PlantOwnershipLog
)
from plugins.media.routes import _process_upload_file
from plugins.utility.models import ImportBatch

@celery_app.task(
    bind=True,
    name="plugins.utility.tasks.import_text_data",
    queue="utility"
)
def import_text_data(self, filepath, import_type, batch_id):
    app = current_app._get_current_object()
    with app.app_context():
        batch = ImportBatch.query.get(batch_id)
        batch.status = "processing"
        batch.started_at = datetime.utcnow()
        db.session.commit()

        is_zip = import_type == "zip"
        tmpdir = None

        try:
            if is_zip:
                tmpdir = tempfile.mkdtemp()
                with zipfile.ZipFile(filepath) as zf:
                    zf.extractall(tmpdir)
                plant_csv = os.path.join(tmpdir, "plants.csv")
                media_csv = os.path.join(tmpdir, "media.csv")
                plant_rows = list(csv.DictReader(
                    open(plant_csv, newline="", encoding="utf-8-sig")
                ))
                media_rows = list(csv.DictReader(
                    open(media_csv, newline="", encoding="utf-8-sig")
                ))
            else:
                plant_rows = list(csv.DictReader(
                    open(filepath, newline="", encoding="utf-8-sig")
                ))
                media_rows = []

            neo = get_neo4j_handler()
            plant_map = {}

            for row in plant_rows:
                common = PlantCommonName.get_or_create(row["Name"])
                scientific = PlantScientificName.get_or_create(
                    row["Scientific Name"], common.id
                )
                raw_mu = row.get("Mother UUID") or None
                mu_for_insert = raw_mu if raw_mu in plant_map else None

                p = Plant(
                    uuid=row["UUID"],
                    common_id=common.id,
                    scientific_id=scientific.id,
                    plant_type=row["Type"],
                    owner_id=batch.user_id,
                    vendor_name=row.get("Vendor Name") or None,
                    price=float(row.get("Price")) if row.get("Price") else None,
                    mother_uuid=mu_for_insert,
                    notes=row.get("Notes") or None,
                    short_id=row.get("Short ID") or None,
                    data_verified=True
                )
                db.session.add(p)
                db.session.flush()
                plant_map[p.uuid] = p.id

                log = PlantOwnershipLog(
                    plant_id=p.id,
                    user_id=batch.user_id,
                    date_acquired=datetime.utcnow(),
                    transferred=False,
                    is_verified=True
                )
                db.session.add(log)

                neo.create_plant_node(p.uuid, row["Name"])
                if raw_mu:
                    neo.create_lineage(child_uuid=p.uuid, parent_uuid=raw_mu)

            db.session.commit()

            for row in plant_rows:
                raw_mu = row.get("Mother UUID") or None
                if raw_mu:
                    Plant.query.filter_by(uuid=row["UUID"]).update({
                        "mother_uuid": raw_mu
                    })
            db.session.commit()

            for mrow in media_rows:
                puuid = mrow["Plant UUID"]
                pid = plant_map.get(puuid)
                if not pid:
                    continue
                rel = mrow["Image Path"].split("uploads/", 1)[-1]
                src = os.path.join(tmpdir, "images", rel) if is_zip else None

                if is_zip and os.path.isfile(src):
                    with open(src, "rb") as f:
                        fs = io.BytesIO(f.read())
                        media = _process_upload_file(
                            file=fs,
                            uploader_id=batch.user_id,
                            plugin="plant",
                            related_id=pid,
                            plant_id=pid
                        )
                    media.uploaded_at = datetime.fromisoformat(mrow["Uploaded At"])
                    media.caption = mrow["Source Type"]
                    db.session.add(media)

            db.session.commit()

            if is_zip and tmpdir:
                shutil.rmtree(tmpdir)
            os.remove(filepath)

            batch.status = "done"
            batch.completed_at = datetime.utcnow()
            db.session.commit()

        except Exception as e:
            app.logger.exception("import_text_data failed")
            batch.status = "failed"
            batch.error = str(e)
            db.session.commit()

        return True
